/* ReSolve V0.08.08c 2016/11/22 solve math expressions using discrete values*/
/* Copyright  2005 - 2016 Valerio Messina http://users.iol.it/efa      */
/* reSolve.c is part of ReSolve
    ReSolve is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ReSolve is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ReSolve. If not, see <http://www.gnu.org/licenses/>. 
      */

/* reSolve.c This is the main source code */

#define SourceVersion "0.08.08c beta"
#define SourceDate    "2016/11/22"

#define _XOPEN_SOURCE 600
#include <stdio.h>      /* printf */
#include <string.h>     /* strcpy, strlen, strcspn */
#include <stdlib.h>     /* qsort, atof, exit */
#include <math.h>       /* pow, fabs */
#include <ctype.h>      /* isdigit */
#include <malloc.h>     /* malloc */
#include "comType.h"    /* common type */
#include "fileIo.h"     /* file Input/Output */
#include "exprParser.h" /* expression parser interface */

#ifdef __MSVCRT__       /* CrossCompile to MinGw target */
#define fsync _commit   /* msvcrt.dll miss fsync, that is present on unix */
#endif                  /* on Win32 try if work using _commit */

/* Up to V0.07.07c before compile, to configure available values, need to set:
    1) 'Series' with xx (96 for E96), or 0 for custom list
    2) Decades 6 or 7 for Exx, ignored with custom list
    3) ListNumber with values quantity in case of custom list
    4) MaxRp 1 or 2 (also test series and parallel in each position)
    5) in custom lists keep only one variable 'baseRdesc[65]' removing numbers after R
    6) in custom lists keep only one variable 'baseR[24]' removing numbers after R
    Optional:
    7) MaxRc as now fixed to 2 variables (letter a,b)
    8) NumberResMax can be kept to 512000 that is compatible with all compilers
*/

#define Series 192 /* Exx: Series E12, E24, (E48), E96 or E192. Use 0 for custom list */
/* Series=E12  & MaxRp=1 & Decades = 5 (30kB), 6 (43kB), 7 (58kB) */
/* Series=E24  & MaxRp=1 & Decades = 5 (115kB), 6 (165kB), 7 (223kB) */
/* Series=E96  & MaxRp=1 & Decades = 5 (1.8MB), 6 (2.6MB), 7 (3.5MB) */
/* Series=E192 & MaxRp=1 & Decades = 5 (7MB), 6 (10MB), 7 (14MB) */
/* Series=E12  & MaxRp=2 & Decades = 5 (106MB), 6 (217MB), 7 (399MB) */
/* Series=E24  & MaxRp=2 & Decades = 1 (3MB), 2 (44MB), 3 (217MB), 4 (676MB), 5 (1.64GB) */
/* Series=E96  & MaxRp=2 & Decades = 1 (676MB) */
#define Decades 7 /* number of decades of interest, normally 6 or 7 */
#if (Series>0) /* standard Exx series */
    #define NumR    (u32)Series*Decades /* number of existant values of resistance */
    #define ListNumber 0 // not used
#else /* custom list */
    //#define ListNumber 57 /* insert here custom list quantity */
    //#define ListNumber 473 /* insert here custom list quantity */
    //#define ListNumber 33 /* insert here custom list quantity */
    //#define ListNumber 14 /* insert here custom list quantity */
    //#define ListNumber 20 /* insert here custom list quantity */
    //#define ListNumber 47 /* insert here custom list quantity */
    //#define ListNumber 14 /* insert here custom list quantity */
    //#define ListNumber 120 /* insert here custom list quantity */
    //#define ListNumber 13 /* insert here custom list quantity */
    #define ListNumber 3 /* insert here custom list quantity */
    //#define ListNumber 12 /* insert here custom list quantity */
    #define NumR   (u32)ListNumber /* number of existant values of resistance */
#endif
#define MaxRp 1 /* max number of resistances supported per position: as now 1 or 2 */
#if (MaxRp==1)
    #define NumV NumR /* number of possible values x each position */
#endif
#if (MaxRp==2)    /* if we have 2 resistors per position ... */
    #define NumV NumR+NumR*NumR+NumR /* number of possible values x each position */
#endif
#define MaxRc 2 /* number of resistances (variables) in the circuit: 2 */
#if (MaxRc==2)   /* as now 2 is the only supported number */
    #define TotV (((u32)NumV)*((u32)NumV)) /* number of values to try */
    // compile&doNotStart with 3'882'988'894 max. Start with 917'797'375 max.
    // compile&doNotStart with   268'402'689 max. Start with  59'954'049 max.
    // RunWell with 59'954'049 max.
#endif
/* default value for formula: reversed high partitor (LM317) */
#define ExpressionDefault "1.25*(1+b/a)        "
/* Formula+spaces:        "        " */
#define DesiredDefault     9 /* default value for desired */

#define NumberResDefault  20  /* default number of results to print */
#define NumberResMax     512000 /* the limit depends on compiler */

#define PRINTBATCH   3
#define PRINTF       4
#define PRINTDEBUG   5
#define PRINTVERBOSE 6
#define DbgLv PRINTF

u08 dbgLev = DbgLv;
u08 dbgLv  = DbgLv;

char*  expr = ExpressionDefault;/* default value for formula: reversed high partitor (LM317) */
double desired;        /* searched value */
u08 Eseries = Series;  /* Exx: Series E12, E24, (E48), E96 or E192. Use 0 for custom list */
u08 decades = Decades; /* number of decades of interest, normally 6 or 7 */
u32 numR = NumR;   /* number of existant values of resistance */
u16 maxRp = MaxRp; /* max number of resistances supported per position: as now 1 or 2 */
u32 numV = NumV;   /* number of possible values x each position */
u16 maxRc = MaxRc; /* number of resistances (variables) in the circuit: 2 */
u32 totV = TotV;   /* number of values to try */
char Exx[24]="";

char   baseR12desc[65] = "standard series E12 @10% tolerance"; // max 64 chars
double baseR12[12] = { 1.0, 1.2, 1.5, 1.8, 2.2, 2.7,   // E12 @10% tolerance, 100ppm. 200pm below 10 Ohm. MIL-PRF55342H (1 to 22 MOhm)
                        3.3, 3.9, 4.7, 5.6, 6.8, 8.2 }; // There was 5.1, 7.5 now removed.

char   baseR24desc[65] = "standard series E24 @5% (and 2%) tolerance"; // max 64 chars
double baseR24[24] = { 1.0, 1.1, 1.2, 1.3, 1.5, 1.6, 1.8, 2.0,   // E24 @5% and 2% tolerance, 100ppm. 200pm below 10 Ohm. MIL-PRF55342H (1 to 22 MOhm)
                       2.2, 2.4, 2.7, 3.0, 3.3, 3.6, 3.9, 4.3,   // M55342M06/VishayRCWPM (0705) down to 1
                       4.7, 5.1, 5.6, 6.2, 6.8, 7.5, 8.2, 9.1 }; // M55342M06/VishayEH    50ppm (20-150k, 10-650k depend on package)

char   baseR96desc[65] = "standard series E96 @1% tolerance"; // max 64 chars
double baseR96[96] = { 10.00, 10.20, 10.50, 10.70, 11.00, 11.30, // E96 @1% tolerance, 100ppm. MIL-PRF55342H (1 to 22 MOhm). 0.1% 15ppm ESCC4001-029/VishayTNPS
                       11.50, 11.80, 12.10, 12.40, 12.70, 13.00, // M55342H06 down to 1 Ohm, VishayRCWPM (0705) down to 10 or 5.62 Ohm (24 values 1st decade)
                       13.30, 13.70, 14.00, 14.30, 14.70, 15.00, // M55342H06 down to 1 Ohm, VishayEH    (0705) down to 20 Ohm(50ppm) or 49.9 Ohm(25ppm)
                       15.40, 15.80, 16.20, 16.50, 16.90, 17.40, // ESCC4001-029/VishayTNPS (0.1% tolerance, 15ppm, 10 to 422 kOhm @0805, to 221 kOhm @0603)
                       17.80, 18.20, 18.70, 19.10, 19.60, 20.00,
                       20.50, 21.00, 21.50, 22.10, 22.60, 23.20, // M55342M06/VishayRCWPM (0705) up to 22 MOhm (33 values last decade)
                       23.70, 24.30, 24.90, 25.50, 26.10, 26.70,
                       27.40, 28.00, 28.70, 29.40, 30.10, 30.90,
                       31.60, 32.40, 33.20, 34.00, 34.80, 35.70,
                       36.50, 37.40, 38.30, 39.20, 40.20, 41.20,
                       42.20, 43.20, 44.20, 45.30, 46.40, 47.50, // ESCC4001-029/VishayTNPS 0.1% 15ppm, up to 422 kOhm 0805 or 221 kOhm 0602
                       48.70, 49.90, 51.10, 52.30, 53.60, 54.90,
                       56.20, 57.60, 59.00, 60.40, 61.90, 63.40,
                       64.90, 66.50, 68.10, 69.80, 71.50, 73.20,
                       75.00, 76.80, 78.70, 80.60, 82.50, 84.50,
                       86.60, 88.70, 90.90, 93.10, 95.30, 97.60 };

char   baseR192desc[65] = "standard series E192 @0.5% (and 0.1%) tolerance"; // max 64 chars
double baseR192[192] = { 10.00, 10.10, 10.20, 10.40, 10.50, 10.60, 10.70, 10.90, // E192 @0.1% tolerance. MIL-PRF55342H (1 to 22 MOhm)
                         11.00, 11.10, 11.30, 11.40, 11.50, 11.70, 11.80, 12.00, // M55342M06 down to 1, VishayRCWPM (100ppm) down to (small:0505)10 Ohm or (0603up)5.62 Ohm
                         12.10, 12.30, 12.40, 12.60, 12.70, 12.90, 13.00, 13.20, // M55342M06 down to 1, VishayEH    down to 20 Ohm(50ppm) or 49.9 Ohm(25ppm)
                         13.30, 13.50, 13.70, 13.80, 14.00, 14.20, 14.30, 14.50,
                         14.70, 14.90, 15.00, 15.20, 15.40, 15.60, 15.80, 16.00,
                         16.20, 16.40, 16.50, 16.70, 16.90, 17.20, 17.40, 17.60,
                         17.80, 18.00, 18.20, 18.40, 18.70, 18.90, 19.10, 19.30,
                         19.60, 19.80, 20.00, 20.30, 20.50, 20.80, 21.00, 21.30,
                         21.50, 21.80, 22.10, 22.30, 22.60, 22.90, 23.20, 23.40, // M55342M06/VishayRCWPM (0705) up to 22 MOhm
                         23.70, 24.00, 24.30, 24.60, 24.90, 25.20, 25.50, 25.80,
                         26.10, 26.40, 26.70, 27.10, 27.40, 27.70, 28.00, 28.40,
                         28.70, 29.10, 29.40, 29.80, 30.10, 30.50, 30.90, 31.20,
                         31.60, 32.00, 32.40, 32.80, 33.20, 33.60, 34.00, 34.40,
                         34.80, 35.20, 35.70, 36.10, 36.50, 37.00, 37.40, 37.90,
                         38.30, 38.80, 39.20, 39.70, 40.20, 40.70, 41.20, 41.70,
                         42.20, 42.70, 43.20, 43.70, 44.20, 44.80, 45.30, 45.90,
                         46.40, 47.00, 47.50, 48.10, 48.70, 49.30, 49.90, 50.50,
                         51.10, 51.70, 52.30, 53.00, 53.60, 54.20, 54.90, 55.60,
                         56.20, 56.90, 57.60, 58.30, 59.00, 59.70, 60.40, 61.20,
                         61.90, 62.60, 63.40, 64.20, 64.90, 65.70, 66.50, 67.30,
                         68.10, 69.00, 69.80, 70.60, 71.50, 72.30, 73.20, 74.10,
                         75.00, 75.90, 76.80, 77.70, 78.70, 79.60, 80.60, 81.60,
                         82.50, 83.50, 84.50, 85.60, 86.60, 87.60, 88.70, 89.80,
                         90.90, 92.00, 93.10, 94.20, 95.30, 96.50, 97.60, 98.80 };

char   baseR1desc[65] = "custom list of 57 values @1% tolerance: TH @ RS"; // max 64 chars
double baseR1[57] = {    470,   510,   560,   620,   680,   750,   820,   910,
                        1000,  1100,  1200,  1300,  1500,  1600,  1800,
                        2000,  2200,  2400,  2700,  3000,  3300,  3600,  3900,
                        4300,  4700,  5100,  5600,  6200,  6800,  7500,  8200, 9100,
                       10000, 11000, 12000, 13000, 15000, 16000, 18000,
                       20000, 22000, 24000, 27000, 30000, 33000, 36000, 39000,
                       43000, 47000, 51000, 56000, 62000, 68000,
                       75000, 82000, 91000, 100000 }; // custom list of 57 values @1% tolerance: TH @ RS

char   baseR2desc[65] = "custom list of 473 values @0.1% tolerance: 10ppm,0805,Tyco/RS"; // max 64 chars
double baseR2[473] = {  5.11, 5.23, 5.36, 5.49, 5.62, 5.76, 6.04, 6.19, 6.34, 6.49, 6.65, 6.81, 6.98, // missing on ESCC4001-029
                        7.15, 7.32, 7.68, 7.87, 8.06, 8.25, 8.45, 8.66, 8.87, 9.09, 9.31, 9.53, 9.76, // missing on ESCC4001-029
                          10, 10.2, 10.5, 10.7, 11, 11.3, 11.5, 11.8, 12.1, 12.4, 12.7, 13, 13.3, 13.7,
                          14, 14.3, 14.7, 15, 15.4, 15.8, 16.2, 16.5, 16.9, 17.4, 17.8, 18.2, 18.7,
                          19.1, 19.6, 20, 20.5, 21, 21.5, 22.1, 22.6, 23.2, 23.7, 24.3, 24.9, 25.5,
                          26.1, 26.7, 27.4, 28, 28.7, 29.4, 30.1, 30.9, 31.6, 32.4, 33.2, 34, 34.8,
                          35.7, 36.5, 37.4, 38.3, 39.2, 40.2, 41.2, 42.2, 43.2, 44.2, 45.3, 46.4, 47.5,
                          48.7, 49.9, 51.1, 52.3, 53.6, 54.9, 56.2, 57.6, 59, 60.4, 61.9, 63.4, 64.9,
                          66.5, 68.1, 69.8, 71.5, 73.2, 75, 76.8, 78.7, 80.6, 82.5, 84.5, 86.6, 88.7,
                          90.9, 93.1, 95.3, 97.6, 100, 102, 105, 107, 110, 113, 115, 118, 121, 124,
                           127, 130, 133, 137, 140, 143, 147, 150, 154, 158, 162, 165, 169, 178, 182,
                           187, 191, 196, 200, 205, 210, 215, 221, 226, 232, 237, 243, 249, 255, 261,
                           267, 274, 280, 287, 294, 301, 309, 316, 324, 332, 340, 348, 357, 365, 374,
                           383, 392, 402, 412, 422, 432, 442, 453, 464, 475, 487, 499, 511, 523, 536,
                           549, 562, 576, 590, 604, 619, 634, 649, 665, 681, 698, 715, 732, 750, 768,
                           787, 806, 825, 845, 866, 887, 909, 931, 953, 976, 1000, 1020, 1050, 1070,
                          1100, 1130, 1150, 1180, 1210, 1240, 1270, 1300, 1330, 1370, 1400, 1430, 1470,
                          1500, 1540, 1580, 1620, 1650, 1690, 1740, 1780, 1820, 1870, 1910, 1960, 2000,
                          2050, 2100, 2150, 2210, 2260, 2320, 2370, 2430, 2490, 2550, 2610, 2670, 2740,
                          2800, 2870, 2940, 3010, 3090, 3160, 3240, 3320, 3400, 3480, 3570, 3650, 3740,
                          3830, 3920, 4020, 4120, 4220, 4320, 4420, 4530, 4640, 4750, 4870, 4990, 5110,
                          5230, 5360, 5490, 5620, 5760, 5900, 6040, 6190, 6340, 6490, 6650, 6810, 6980,
                          7150, 7320, 7500, 7680, 7870, 8060, 8250, 8450, 8660, 8870, 9090, 9310, 9530,
                          9760, 10000, 10200, 10500, 10700, 11000, 11300, 11500, 11800, 12100, 12400,
                         12700, 13000, 13300, 13700, 14000, 14300, 14700, 15000, 15400, 15800, 16200,
                         16500, 16900, 17400, 17800, 18200, 18700, 19100, 19600, 20000, 20500, 21000,
                         21500, 22100, 22600, 23200, 23700, 24300, 24900, 25500, 26100, 26700, 28000,
                         28700, 29400, 30100, 30900, 31600, 32400, 33200, 34000, 34800, 35700, 36500,
                         37400, 38300, 39200, 40200, 41200, 42200, 43200, 44200, 45300, 46400, 47500,
                         48700, 49900, 51100, 52300, 53600, 54900, 56200, 57600, 59000, 60400, 61900,
                         63400, 64900, 66500, 68100, 69800, 71500, 73200, 75000, 76800, 78700, 80600,
                         82500, 84500, 86600, 88700, 90900, 93100, 95300, 97600, 100000, 102000,
                        105000, 107000, 110000, 113000, 115000, 118000, 121000, 124000, 127000,
                        130000, 133000, 137000, 140000, 143000, 147000, 150000, 154000, 158000,
                        162000, 165000, 169000, 174000, 178000, 182000, 187000, 191000, 196000,
                        200000, 205000, 210000, 215000, 221000, 226000, 232000, 237000, 243000,
                        249000, 255000, 261000, 267000, 274000, 280000, 287000, 294000, 301000,
                        309000, 316000, 324000, 332000, 340000, 348000, 357000, 365000, 374000,
                        383000, 392000, 402000, 412000, 422000, 432000, 442000, 453000, 464000 // ESCC4001-029 up to 422k only
                      }; // custom list of 473 values @0.1% tolerance: 10ppm, SMD0805, TycoElectronics(RS)

char   baseR3desc[65] = "custom list of 33 values RNC90 @0.1% tolerance"; // max 64 chars
double baseR3[33] = {    20,    40,    45,  49.9,    50,  50.5,  98.8, 98.8,   100,
                        101,   200,   500,  1510,  2250,  2500,  3000,   4125,  8000,
                       9880, 10100, 13000, 22500, 25000, 37500, 39800, 50000, 50500,
                      54900, 54900, 55000, 67000, 80600, 98800
                    }; // custom list of 33 values RNC90 @0.1% tolerance
                       // are the only precision values available in labs

char   baseR4desc[65] = "custom list of 14 values ESCC4001-029 @0.1% tolerance"; // max 64 chars
double baseR4[14] = { 28, 50.5, 499, 1000, 1070, 1500, 2000, 2740, 3160, 3740, 10000, 10500, 31200, 100000
                     }; // custom list of 14 values ESCC4001-029 @0.1% tolerance
                        // are the only precision values available in DCL

char   baseR5desc[65] = "custom list of 20 values ESCC4001-029 @0.1% tolerance"; // max 64 chars
double baseR5[20] = {  28, 51.1, 1070, 1500, 1820, 2000, 2740, 3160, 3740, 3920,
                      4320, 4990, 6040, 10000, 10500, 13300, 18200, 47500, 49900, 90900
                     }; // custom list of 20 values ESCC4001-029 @0.1% tolerance
                        // are the only precision values available in store

char   baseR6desc[65] = "custom list of 47 values ESCC4001-029 @0.1% tolerance"; // max 64 chars
double baseR6[47] = { 5.49, 10, 28, 28.7, 37.4, 51.1, 75, 78.7, 93.1, 100, 102, 105, 118,
                       147, 162, 301, 309, 402, 549, 845, 1050, 1070, 1100, 1430, 1500,
                       1820, 2000, 2490, 2740, 3160, 3740, 3920, 4320, 4990, 6040, 6650,
                       7150, 8060, 8450, 9760, 10000, 10500, 13300, 18200, 47500, 49900, 90900
                     }; // custom list of 47 values ESCC4001-029 @0.1% tolerance
                        // are the only precision values available in store

char   baseR7desc[65] = "custom list of 14 values ESCC4001-029 @0.1% tolerance"; // max 64 chars
double baseR7[14] = { 10, 24.9, 49.9, 75, 100, 249, 499, 750, 1000, 2000, 2490, 2550, 10500, 28700 }; // custom list of 12 values ESCC4001-029 @0.1% tolerance
                        // are the only precision values available for trimming

char   baseR8desc[65] = "custom list of 24 values ESCC4001-029 @0.1% tolerance"; // max 64 chars
double baseR8[24] = { 10, 24.9, 49.9, 75, 100, 249, 499, 750, 1000, 1240, 2000, 2430,
                       2490, 2550, 3160, 4420, 4640, 6040, 10000, 10500, 14700, 28700,
                       100000, 200000 }; // custom list of 24 values ESCC4001-029 @0.1% tolerance
                                        // are the only precision values available in DCL

char   baseR9desc[65] = "custom list of 120 values @1% tolerance: 100ppm,0805,Tyco/RS"; // max 64 chars
double baseR9[120] = { 5.62, 5.76, 5.90, 6.04, 6.19, 6.34, 6.49, 6.65, 6.81, 6.98, 7.15, 7.32,
                        7.50, 7.68, 7.87, 8.06, 8.25, 8.45, 8.66, 8.87, 9.09, 9.31, 9.53, 9.76,
                        10.00, 10.20, 10.50, 10.70, 11.00, 11.30, 11.50, 11.80, 12.10, 12.40,
                        12.70, 13.00, 13.30, 13.70, 14.00, 14.30, 14.70, 15.00, 15.40, 15.80,
                        16.20, 16.50, 16.90, 17.40, 17.80, 18.20, 18.70, 19.10, 19.60, 20.00,
                        20.50, 21.00, 21.50, 22.10, 22.60, 23.20, 23.70, 24.30, 24.90, 25.50,
                        26.10, 26.70, 27.40, 28.00, 28.70, 29.40, 30.10, 30.90, 31.60, 32.40,
                        33.20, 34.00, 34.80, 35.70, 36.50, 37.40, 38.30, 39.20, 40.20, 41.20,
                        42.20, 43.20, 44.20, 45.30, 46.40, 47.50, 48.70, 49.90, 51.10, 52.30,
                        53.60, 54.90, 56.20, 57.60, 59.00, 60.40, 61.90, 63.40, 64.90, 66.50,
                        68.10, 69.80, 71.50, 73.20, 75.00, 76.80, 78.70, 80.60, 82.50, 84.50,
                        86.60, 88.70, 90.90, 93.10, 95.30, 97.60 }; // M55342M06 down to 1, VishayRCWPM-575 (0805) down to 5.62 Ohm

char   baseR10desc[65] = "custom list of 13 values ESCC4001-027 @0.5% tolerance"; // max 64 chars
double baseR10[13] = { 0.010, 0.027, 0.047, 0.050, 0.056, 0.100, 0.200, 0.220, 0.390, 0.470, 0.500, 0.820, 1.000 }; // custom list of 13 values ESCC4001-027 @0.5% tolerance
                        // are the only precision values available for current sensing

char   baseR11desc[65] = "custom list of 3 values Farnell @1% tolerance"; // max 64 chars
double baseR11[3] = { 0.1, 0.33, 0.47 }; // custom list of 3 values Farnell @1% tolerance
                        // are the only precision values available for current sensing

char   baseR13desc[65] = "custom list of 12 values Farnell @5% tolerance"; // max 64 chars
double baseR13[12] = { 0.1, 0.12, 0.18, 0.22, 0.27, 0.33, 0.39, 0.47, 0.51, 0.56, 0.68, 0.82 }; // custom list of 12 values Farnell @5% tolerance
                        // are the only precision values available for current sensing

u16 listNumber = ListNumber; // custom list quantity

char    baseRdesc[65]=""; // reserve space for 65 chars
double* baseR; // declare vector pointer, will be a vector of double baseR[listNumber]

struct rValuesTy { double* rp; /* will be a vector of values with [maxRp] elements */
                    double  r;        /* resultant value, series or parallel */
                    char    desc[25]; /* description of how is built (series or parallel) */
                  }; /* struct declaration */
struct rValuesTy* rValues; /* pointer to memory for single, series & parallel */

struct resultsTy { u16   pos[MaxRc]; /* positions of each resistance, [maxRc] elements */
                    float delta;      /* distance from desiderata */
                  }; /* struct declaration */
//static struct resultsTy results[totV]; /* allocate memory for results: [(12*7)^2] */
static struct resultsTy* results; /* pointer to memory for results: [(12*7)^2] */

u32 powlmy (u32 base, u32 exp) { /* substitute the pow() in math.h but use u32 */
    register u32 c;
    register u32 res = 1;
    for (c=0; c<exp; c++) {
       res = res * base;
    }
    return (res);
}

/* using one decade values and 'decades', calculate all standard Exx series values */
// use 'baseRxx[Eseries/listNumber]' and generate ==> rValues[pos].r, rValues[pos].rp[p]
s16 calcEseries (void) { // or fill with the custom list of values when Eseries=0
   u08 decade;
   u08 rBase;
   s16 pos = 0; // expected max 7*192=1344
   register double val;
   u08 p;
   if (Eseries>0) { // standard Exx series resistors
      for (decade=0; decade<decades; decade++) {
         for (rBase=0; rBase<Eseries; rBase++) {
            pos = rBase + decade*Eseries;
            switch (Eseries) {
            case (12):
               /*printf ("rBase:%u baseR12[rBase]:%lu decade:%u pos:%ld\n", rBase, baseR12[rBase], decade, pos);*/
               val = baseR12[rBase] * (double)powlmy (10, decade);
               break;
            case (24):
               /*printf ("rBase:%u baseR24[rBase]:%lu decade:%u pos:%ld\n", rBase, baseR12[rBase], decade, pos);*/
               val = baseR24[rBase] * (double)powlmy (10, decade);
               break;
            case (96):
               /*printf ("rBase:%u baseR96[rBase]:%lu decade:%u pos:%ld\n", rBase, baseR12[rBase], decade, pos);*/
               val = baseR96[rBase] * (double)powlmy (10, decade);
               break;
            case (192):
               /*printf ("rBase:%u baseR192[rBase]:%lu decade:%u pos:%ld\n", rBase, baseR12[rBase], decade, pos);*/
               val = baseR192[rBase] * (double)powlmy (10, decade);
               break;
            default:
               printf ("Unsupported Series:%u. Supported are 12, 24, 96 and 192\n", Eseries);
               return (ERROR);
            }
            rValues[pos].r = val;
            for (p=0; p<maxRp; p++) {
               rValues[pos].rp[p] = val;
            }
            //strcpy (rValues[pos].desc, "Exxx series"); // 11 chars
            strcpy (rValues[pos].desc, Exx);
            if (dbgLv>=PRINTDEBUG) printf ("rValue:%g\n", val);
         }
      }
   } else { // custom list of values
      for (pos=0; pos<listNumber; pos++) { // populate with custom values
         val = baseR[pos];
         rValues[pos].r = val;
         for (p=0; p<maxRp; p++) {
            rValues[pos].rp[p] = val;
         }
         strcpy (rValues[pos].desc, "Custom list"); // 11 chars
         if (dbgLv>=PRINTVERBOSE) printf ("rValue:%g\n", val);
      }
   }
   if (dbgLv>=PRINTDEBUG) printf ("pos:%d\n", pos);
   return (pos);
} // s16 calcEseries (void)

/* calculate all possible resistive values using 'MaxRp' resistances */
int calcRvalues (void) { /* in series or parallel, support MaxRp=2 only */
   s16 pos;
   register double val;
   u16 rp1, rp2;
   u08 p;
   /* at first calculate the single values (Eseries) or fill custom values */
   pos = calcEseries ();
   if (pos==ERROR) {
      printf ("calcEseries returned ERROR\n");
      return (ERROR);
   }
   if (dbgLv>=PRINTDEBUG) printf ("pos:%d\n", pos);
   if (maxRp==2) { // as now support only maxRp=2 resistances per position
      /* now calculate all series values with 2 resistances per position */
      pos = numR;
      for (rp1=0; rp1<numR; rp1++) {
         for (rp2=0; rp2<numR; rp2++) {
            if (rp1<rp2) continue; /* triangular matrix */
            val = rValues[rp1].r + rValues[rp2].r;
            rValues[pos].r = val;
            rValues[pos].rp[0] = rValues[rp1].r;
            for (p=1; p<maxRp; p++) {
               rValues[pos].rp[p] = rValues[rp2].r;
            }
            strcpy (rValues[pos].desc, "Series   of"); // 11 chars
/*           printf ("r1:%g + r2:%g = ", rValues[rp1].r, rValues[rp2].r);
            printf ("rValue:%g\n", val);*/
            pos++;
         }
      }
      if (dbgLv>=PRINTDEBUG) printf ("pos:%d\n", pos);
      /* now calculate all parallel values with 2 resistances per position */
      for (rp1=0; rp1<numR; rp1++) {
         for (rp2=0; rp2<numR; rp2++) {
            if (rp1<rp2) continue; /* triangular matrix */
            val = rValues[rp1].r * rValues[rp2].r / (rValues[rp1].r + rValues[rp2].r);
            rValues[pos].r = val;
            rValues[pos].rp[0] = rValues[rp1].r;
            for (p=1; p<maxRp; p++) {
               rValues[pos].rp[p] = rValues[rp2].r;
            }
            strcpy (rValues[pos].desc, "Parallel of"); // 11 chars
/*           printf ("r1:%g // r2:%g = ", rValues[rp1].r, rValues[rp2].r);
            printf ("rValue:%g\n", val);*/
            pos++;
         }
      }
   }
   if (maxRp>2) {
      printf ("Unsupported maxRp:%u. Supported maxRp 1 or 2\n", maxRp);
      return (ERROR);
   }
   if (dbgLv>=PRINTDEBUG) printf ("pos:%d\n", pos);
   return (OK);
} // int calcRvalues (void)

#include <unistd.h> // fsync

/* calculate all formula results using 'maxRc' resistances */
int calcFvalues (void) {
   u32 rc1, rc2;
   /*float res[maxRc];*/ /* single case, normally 2 resistances */
   register double val;
   register double delta = 0;
   register u32 pos = 0;
   if (dbgLv>=PRINTDEBUG) printf ("numV:%u, totV:%u\n", numV, totV);
   if (dbgLv>=PRINTDEBUG) printf ("expr:'%s'\n", expr);
   if (dbgLv>=PRINTF) printf ("sol="); // here print something for user feedback (take time)
   for (rc1=0; rc1<numV; rc1++) {
      exprVarsParser[0] = rValues[rc1].r; /* estract single case to try */
      if (dbgLv>=PRINTDEBUG) printf ("exprVarsParser[0]:%g\n", exprVarsParser[0]);
      for (rc2=0; rc2<numV; rc2++) {
         exprVarsParser[1] = rValues[rc2].r; /* estract single case to try */
         if (dbgLv>=PRINTDEBUG) printf ("exprVarsParser[1]:%g\n", exprVarsParser[1]);
         val = evalExprParser (expr);
         /*if (desired >= val) delta = desired - val;
         if (val > desired) delta = val - desired;*/
         delta = desired - val;
         /*printf ("rc1:%u rc2:%u pos:%lu val:%g delta:%g\n", rc1, rc2, pos, val, delta);*/
         results[pos].pos[0] = rc1;
         results[pos].pos[1] = rc2;
         /*results[pos].val    = val;*/
         results[pos].delta  = delta;
         pos++;
      }
      if (rc1%1000==0) {
         if (dbgLv>=PRINTF) printf ("%u,", rc1*numV);
         fflush (NULL); // user space stdout flush
         fsync (1);   // kernel space stdout flush
      }
   }
   if (dbgLv>=PRINTF) printf ("%u\n", rc1*numV-1);
   return (OK);
} // int calcFvalues (void) {

s32 tot; // used for communication between 'structQuickSort' & 'qsStruct'

/* QuickSort for vector of structs of type resultsTy. */
/* Sorting criteria: field 'abs(delta)' */
void qsStruct (struct resultsTy results[], s32 left, s32 right) {
   register s32 l, r, m; /* position left, right and median */
   float  mp; /* will contain data in center position */
   struct resultsTy temp;            /* temp struct for swap */
   static u32 qs=0;
   if (qs%6000000==0) {
      if (dbgLv>=PRINTF) printf ("%u,", qs);
      fflush (NULL); // user space stdout flush
      fsync (1);   // kernel space stdout flush
   }
   qs++;
   if (dbgLv>=PRINTDEBUG) {
      for (s32 c=0; c<tot; c++) {
         printf ("s[%2d]=%11G\n", c, results[c].delta);
      }
   }
   if (dbgLv>=PRINTDEBUG) printf ("left:%9d right:%9d\n", left, right);
   l = left; r = right;
   m = (left+right)/2; /* center position rounded down */
   mp = fabs(results[m].delta); /* take center element in module */
   if (dbgLv>=PRINTDEBUG) printf ("meanPos(left+right)/2:%9d meanValue_delta[meanPos]:%11G\n", m, mp);
   do { /* at least once, until is true (l<=r) */
      if (dbgLv>=PRINTDEBUG) printf ("l:%9d r:%9d\n", l, r);
      /* find first element at left <= of center element */
      while ( ((mp-fabs(results[l].delta))<0) && (l<right) ) l++; /* test, if true exec */
      /* find first element at right > of center element */
      while ( ((mp-fabs(results[r].delta))>0) && (r>left ) ) r--; /* test, if true exec */
      if (dbgLv>=PRINTDEBUG) printf ("limits: value[l=%9d]=%11G value[r=%9d]=%11G\n", l, results[l].delta, r, results[r].delta);
      if (l <= r) { /* if not crossed, swap */
         if (dbgLv>=PRINTDEBUG) printf ("swap: l:%9d r:%9d\n", l, r);
         temp = results[l];
         results[l] = results[r];
         results[r] = temp;
         l++; r--;
      }
      if (dbgLv>=PRINTDEBUG) {
         for (s32 c=0; c<tot; c++) {
            printf (" [%2d]=%11G\n", c, results[c].delta);
         }
      }
   } while (l <= r);
   if (dbgLv>=PRINTVERBOSE) {
      for (s32 c=left; c<right; c++) {
         printf ("value[%9d]=%11G\n", c, results[c].delta);
      }
   }
   if (left < r) {
      if (dbgLv>=PRINTDEBUG) printf ("analisys: value[left=%9d]=%11G value[r=%9d]=%11G\n", left, results[left].delta, r, results[r].delta);
      if (dbgLv>=PRINTDEBUG) printf ("\n");
      qsStruct (results, left , r);
   }
   if (l < right) {
      if (dbgLv>=PRINTDEBUG) printf ("analisys: value[l=%9d]=%11G value[right=%9d]=%11G\n", l, results[l].delta, right, results[right].delta);
      if (dbgLv>=PRINTDEBUG) printf ("\n");
      qsStruct (results, l, right);
   }
   if (dbgLv>=PRINTDEBUG) printf ("\n");
} // void qsStruct (struct resultsTy results[], s32 left, s32 right)

/* QuickSort for vector of structs of type resultsTy, using field 'abs(delta)' */
/* put in results[0] the greatest and in results[totNumber-1] the smallest */
int structQuickSort (struct resultsTy results[], s32 totNumber) {
   if (totNumber==0) return (ERROR);
   tot = totNumber;
   if (dbgLv>=PRINTF) {
      printf ("qs="); // here print something for user feedback (take time)
      fflush (NULL); // user space stdout flush
      fsync (1);   // kernel space stdout flush
   }
   qsStruct (results, 0UL, totNumber-1);
   if (dbgLv>=PRINTF) printf ("\n");
   if (dbgLv>=PRINTDEBUG) {
      for (s32 c=0; c<tot; c++) {
         printf ("f[%2d]=%11G\n", c, results[c].delta);
      }
   }
   return (OK);
} // int structQuickSort (struct resultsTy results[], s32 totNumber)

/* print last 'numBestRes=totV-first' results */
int showVal (u32 first) { // solutions with up to 4 resistors
   u32 pos;
   int cmp0, cmp1;
   for (pos=first; pos<totV; pos++) { // 'totV-1' is the best solution
      /*printf ("pos:%9lu", pos);*/
      cmp0 = (strcmp (rValues[results[pos].pos[0]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[0]].desc, Exx          )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      cmp1 = (strcmp (rValues[results[pos].pos[1]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[1]].desc, Exx          )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      printf ("a:%11G b:%11G", rValues[results[pos].pos[0]].r, rValues[results[pos].pos[1]].r);
      printf ("   val:%11G   delta:%11G", desired-results[pos].delta, fabs(results[pos].delta));
      printf (" e%%:%6.3G\n", fabs(results[pos].delta/desired*100));
      if (maxRp==2) { // maybe a series or parallel value
         if (cmp0==0) { // single resistors
            printf ("a:%-12s:%8g             ", rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0]);
         } else { // 2 resistors in series or parallel
            printf ("a:%-12s:%8g & %8g  " , rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0], rValues[results[pos].pos[0]].rp[1]);
         }
         if (cmp1==0) { // single resistors
            printf ("b:%-12s:%8g", rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0]);
         } else { // 2 resistors in series or parallel
            printf ("b:%-12s:%8g & %8g ", rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0], rValues[results[pos].pos[1]].rp[1]);
         }
         printf ("\n");
      }
   }
   return (OK);
} // int showVal (u32 first)

/* print best 'numBestRes' results */
int showVal4 (u32 numBestRes) { // Solutions with 4 resistors
   s32 pos, best;
   u32 count=0;
   int cmp0, cmp1;
   u32 res[NumberResMax];
   //printf ("numBestRes:%ld NumberResMax:%d\n", numBestRes, NumberResMax);
   if (numBestRes>NumberResMax) numBestRes=NumberResMax;
   //printf ("numBestRes:%ld NumberResMax:%d\n", numBestRes, NumberResMax);
   for (pos=totV-1; pos>=0; pos--) { // [totV] is the best solution
      /*printf ("pos:%9lu", pos);*/
      if (count>=numBestRes) break; // printed 'numBestRes' results
      cmp0 = (strcmp (rValues[results[pos].pos[0]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[0]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      cmp1 = (strcmp (rValues[results[pos].pos[1]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[1]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      if (cmp0==0 || cmp1==0) continue; // skip when one is 0 (so skip when are less than 4 resistances, do when both are series/parallel)
      res[count]=pos;
      //printf ("4 added pos:%ld\n", pos);
      count++;
   }
   for (best=count-1; best>=0; best--) { // [0] is the best solution
      pos=res[best];
      cmp0 = (strcmp (rValues[results[pos].pos[0]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[0]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      cmp1 = (strcmp (rValues[results[pos].pos[1]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[1]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      printf ("a:%11G b:%11G", rValues[results[pos].pos[0]].r, rValues[results[pos].pos[1]].r);
      printf ("   val:%11G   delta:%11G", desired-results[pos].delta, fabs(results[pos].delta));
      printf (" e%%:%6.3G\n", fabs(results[pos].delta/desired*100));
      if (maxRp==2) { // maybe a series or parallel value
         if (cmp0==0) { // single resistors
            printf ("a:%-12s:%8g             ", rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0]);
         } else { // 2 resistors in series or parallel
            printf ("a:%-12s:%8g & %8g  "  , rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0], rValues[results[pos].pos[0]].rp[1]);
         }
         if (cmp1==0) { // single resistors
            printf ("b:%-12s:%8g", rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0]);
         } else { // 2 resistors in series or parallel
            printf ("b:%-12s:%8g & %8g ", rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0], rValues[results[pos].pos[1]].rp[1]);
         }
         printf ("\n");
      }
   }
   return (OK);
} // int showVal4 (u32 numBestRes)

/* print best 'numBestRes' results */
int showVal3 (u32 numBestRes) { // Solutions with 3 resistors
   s32 pos, best;
   u32 count=0;
   int cmp0, cmp1;
   u32 res[NumberResMax];
   if (numBestRes>NumberResMax) numBestRes=NumberResMax;
   for (pos=totV-1; pos>=0; pos--) { // [totV] is the best solution
      if (count>=numBestRes) break; // printed 'numBestRes' results
      cmp0 = (strcmp (rValues[results[pos].pos[0]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[0]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      cmp1 = (strcmp (rValues[results[pos].pos[1]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[1]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      if (cmp0==cmp1) continue; // skip when both 0 or both 1 (both Arb/Exx or both series/parallel, so do when are 3 resistances)
      res[count]=pos;
      //printf ("3 added pos:%ld\n", pos);
      count++;
   }
   for (best=count-1; best>=0; best--) { // [0] is the best solution
      pos=res[best];
      cmp0 = (strcmp (rValues[results[pos].pos[0]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[0]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      cmp1 = (strcmp (rValues[results[pos].pos[1]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[1]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      printf ("a:%11G b:%11G", rValues[results[pos].pos[0]].r, rValues[results[pos].pos[1]].r);
      printf ("   val:%11G   delta:%11G", desired-results[pos].delta, fabs(results[pos].delta));
      printf (" e%%:%6.3G\n", fabs(results[pos].delta/desired*100));
      if (maxRp==2) {
         if (cmp0==0) { // single resistors
            printf ("a:%-12s:%8g             ", rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0]);
         } else { // 2 resistors in series or parallel
            printf ("a:%-12s:%8g & %8g  " , rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0], rValues[results[pos].pos[0]].rp[1]);
         }
         if (cmp1==0) { // single resistors
            printf ("b:%-12s:%8g", rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0]);
         } else { // 2 resistors in series or parallel
            printf ("b:%-12s:%8g & %8g ", rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0], rValues[results[pos].pos[1]].rp[1]);
         }
         printf ("\n");
      }
   }
   return (OK);
} // int showVal3 (u32 numBestRes)

/* print best 'numBestRes' results */
int showVal2 (u32 numBestRes) { // Solutions with 2 resistors
   s32 pos, best;
   u32 count=0;
   int cmp0, cmp1;
   u32 res[NumberResMax];
   if (numBestRes>NumberResMax) numBestRes=NumberResMax;
   for (pos=totV-1; pos>=0; pos--) { // [totV] is the best solution
      if (count>=numBestRes) break; // printed 'numBestRes' results
      cmp0 = (strcmp (rValues[results[pos].pos[0]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[0]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      cmp1 = (strcmp (rValues[results[pos].pos[1]].desc, "Custom list")!=0) &&
             (strcmp (rValues[results[pos].pos[1]].desc, Exx )!=0);   // 0 when single (Exx or custom), 1 when two (series or //)
      if (cmp0!=0 || cmp1!=0) continue; // skip when one is 1 (so skip when are more than 2 resistances, do when both are Exx/custom)
      res[count]=pos;
      //printf ("2 added pos:%ld\n", pos);
      count++;
   }
   for (best=count-1; best>=0; best--) { // [0] is the best solution
      pos=res[best];
      printf ("a:%11G b:%11G", rValues[results[pos].pos[0]].r, rValues[results[pos].pos[1]].r);
      printf ("   val:%11G   delta:%11G", desired-results[pos].delta, fabs(results[pos].delta));
      printf (" e%%:%6.3G\n", fabs(results[pos].delta/desired*100));
      if (maxRp==2) { // this is just a repetition of 'a' and 'b' as are Custom list
         printf ("a:%-12s:%8g             ", rValues[results[pos].pos[0]].desc, rValues[results[pos].pos[0]].rp[0]);
         printf ("b:%-12s:%8g"       , rValues[results[pos].pos[1]].desc, rValues[results[pos].pos[1]].rp[0]);
         printf ("\n");
      }
   }
   return (OK);
} // int showVal2 (u32 numBestRes)

void showHead (void) {
   printf ("ReSolve v%s %s by Valerio Messina users.iol.it/efa\n", SourceVersion, SourceDate);
   printf ("\n");
}

void showHelp (float allocatedMB) {
   showHead ();
   /*printf ("Syntax: reSolve desiredFloatValue [numberOfResults]\n");*/
   printf ("Syntax: reSolve desiredFloatValue [\"function(a,b)\" [numberOfResults]]\n");
   printf ("\n");
   printf ("functions support: numbers, variables a-b, operators +,-,*,/,%%,^,(,)\n");
   printf ("if (function==NULL) function=%s\n", ExpressionDefault);
   printf ("if (!numberOfResults) numberOfResults=%u\n", NumberResDefault);
   printf ("\n");
   if (Eseries>0) { // standard Exx series resistors
      switch (Eseries) {
      case (12):
         strcpy (baseRdesc, baseR12desc);
         break;
      case (24):
         strcpy (baseRdesc, baseR24desc);
         break;
      case (96):
         strcpy (baseRdesc, baseR96desc);
         break;
      case (192):
         strcpy (baseRdesc, baseR192desc);
         break;
      default:
         printf ("Unsupported Series:%u. Supported are 12, 24, 96 and 192\n", Eseries);
         return;
      }
   }
   printf ("Values list: '%s'", baseRdesc);
   if (Eseries>0) printf (" with %u decades", decades);
   printf ("\n");
   printf ("numR:%u, numV:%u, totV:%u\n", numR, numV, totV);
   printf ("%u possible values with max:%u resistors per position\n", numV, maxRp);
   //printf ("WARN: may allocate about %lu MB of RAM !\n", 220*powlmy(2, decades-5));
   if (allocatedMB>200) printf ("WARN: may allocate about %.2f MB of RAM !\n", allocatedMB);
   printf ("\n");
} // showHelp (float allocatedMB)

int fillConfigVars (void) { // load and check users config file
   char fileConf[]="reSolveConf.txt";
   char* bufferPtr;
   off_t bufSize;
   char paramName[LineLen];
   char paramValue[LineLen];
   int  ret;
   bufSize=readFile (fileConf, &bufferPtr);
   if (bufSize==ERROR) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot read file:%s\n", __FUNCTION__, fileConf);
      return (ERROR);
   }
   if (bufferPtr==NULL) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: returned NULL pointer\n", __FUNCTION__);
      return (ERROR);
   }
   //printf ("bufSize=%lld\n", (s64)bufSize);
   //printf ("bufferPtr:'\n%s\n':bufferPtr\n", bufferPtr);
   //printf ("\n");

   // reading user configured parameters ...
   char* endPtr;
   strcpy (paramName, "expr");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTWARN) printf ("WARN %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
   } else {
      //printf ("expr='%s'\n", paramValue);
      expr = malloc (52);
      if (!expr) {
         printf ("Dynamic allocation of:%u Bytes failed:0x%8p\n", 52, expr);
         return (ERROR);
      }
      endPtr = strcpy (expr, paramValue);
      if (endPtr==NULL) {
         if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot copy paramValue='%s'\n", __FUNCTION__, paramValue);
         return (ERROR);
      }
   }
   if (dbgLev>=PRINTDEBUG) printf ("expr='%s'\n", expr);
   strcpy (paramName, "desired");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTWARN) printf ("WARN %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
   } else {
      //printf ("desired='%s'\n", paramValue);
      desired = strtol (paramValue, &endPtr, 10);
      if (endPtr==paramValue) {
         if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find digits in desired='%s'\n", __FUNCTION__, paramValue);
         return (ERROR);
      }
   }
   if (dbgLev>=PRINTDEBUG) printf ("desired=%g\n", desired);
   strcpy (paramName, "Eseries");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   //printf ("Eseries='%s'\n", paramValue);
   Eseries = strtol (paramValue, &endPtr, 10);
   if (endPtr==paramValue) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find digits in Eseries='%s'\n", __FUNCTION__, paramValue);
      return (ERROR);
   }
   if (dbgLev>=PRINTDEBUG) printf ("Eseries=%u\n", Eseries);
   strcpy (paramName, "decades");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   //printf ("decades='%s'\n", paramValue);
   decades = strtol (paramValue, &endPtr, 10);
   if (endPtr==paramValue) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find digits in decades='%s'\n", __FUNCTION__, paramValue);
      return (ERROR);
   }
   if (dbgLev>=PRINTDEBUG) printf ("decades=%u\n", decades);
   strcpy (paramName, "listNumber");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   //printf ("listNumber='%s'\n", paramValue);
   listNumber = strtol (paramValue, &endPtr, 10);
   if (endPtr==paramValue) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find digits in listNumber='%s'\n", __FUNCTION__, paramValue);
      return (ERROR);
   }
   if (dbgLev>=PRINTDEBUG) printf ("listNumber=%u\n", listNumber);
   strcpy (paramName, "maxRp");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   //printf ("maxRp='%s'\n", paramValue);
   maxRp = strtol (paramValue, &endPtr, 10);
   if (endPtr==paramValue) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find digits in maxRp='%s'\n", __FUNCTION__, paramValue);
      return (ERROR);
   }
   if (dbgLev>=PRINTDEBUG) printf ("maxRp=%u\n", maxRp);
   strcpy (paramName, "maxRc");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   //printf ("maxRc='%s'\n", paramValue);
   maxRc = strtol (paramValue, &endPtr, 10);
   if (endPtr==paramValue) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find digits in maxRc='%s'\n", __FUNCTION__, paramValue);
      return (ERROR);
   }
   if (dbgLev>=PRINTDEBUG) printf ("maxRc=%u\n", maxRc);
   strcpy (paramName, "baseRdesc");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   //printf ("baseRdesc='%s'\n", paramValue);
   endPtr = strcpy (baseRdesc, paramValue);
   if (endPtr==NULL) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot copy paramValue='%s'\n", __FUNCTION__, paramValue);
      return (ERROR);
   }
   if (dbgLev>=PRINTDEBUG) printf ("baseRdesc='%s'\n", baseRdesc);

   strcpy (paramName, "baseR");
   ret=parseConf (bufferPtr, paramName, paramValue);
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: cannot find:%s in config file\n", __FUNCTION__, paramName);
      return (ERROR);
   }
   /*for (u16 p=0; p<16; p++) {
      if (dbgLev>=PRINTDEBUG) printf ("@:%p paramValue[%u]=0x%02x='%c'\n", &paramValue[p], p, paramValue[p], paramValue[p]);
   }*/
   //dbgLev=PRINTDEBUG;
   u16 size=paramValue[16]; // copy the size of baseR from fixed position
   if (dbgLev>=PRINTDEBUG) printf ("array size:%u=0x%x\n", size, size);
   if (listNumber!=size) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: listNumber:%u <> baseR[size]:%u in config file\n", __FUNCTION__, listNumber, size);
      return (ERROR);
   }
   paramValue[16]='\0'; // clear size overlapping with string terminator

   unsigned long int hex;
   sscanf (paramValue, "%16lx", &hex);
   if (dbgLev>=PRINTDEBUG) printf ("hex:%lu=0x%16lx\n", hex, hex);
   double* vectorPtr;
   vectorPtr=(void*)(size_t)hex;
   //double baseR[size]; // allocate space for custom values
   //double* baseR; // declare vector pointer
   baseR=malloc(size*sizeof(double)); // allocate space for custom values
   if (!baseR) {
      printf ("Dynamic allocation of:%zu Bytes failed:0x%8p\n", size*sizeof(double), baseR);
      return (ERROR);
   }
   // now can use global baseR[size] for custom values
   for (u16 p=0; p<size; p++) {
      if (dbgLev>=PRINTDEBUG) printf ("vectorPtr[%u]:%g\n", p, vectorPtr[p]);
      baseR[p]=vectorPtr[p]; // fill vector of custom values
   }
   free (vectorPtr);
   if (dbgLev>=PRINTDEBUG) printf ("baseR[%u]:", size);
   for (u16 p=0; p<size; p++) {
      if (dbgLev>=PRINTDEBUG) printf ("%g, ", baseR[p]);
   }
   if (dbgLev>=PRINTDEBUG) printf ("\n");
   //dbgLev=PRINTF;

   //printf ("Freeing buffer ...\n");
   free (bufferPtr);
   // check supported user request
   //Eseries=192; // possible values are: 12, 24, 48, 96, 192. Use 0 for custom list
   switch (Eseries) {
   case 0: case 12: case 24: case 48: case 96: case 192:
      if (dbgLev>=PRINTDEBUG) printf ("supported Eseries\n");
      break;
   default:
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: unsupported Eseries:%u\n", __FUNCTION__, Eseries);
      return (ERROR);
   }
   strcpy (Exx, "E");
   char series[4];
   sprintf (series, "%u", Eseries);
   strcat (Exx, series);
   strcat (Exx, " series"); // 11 chars
   //decades=7; // ignored when Eseries=0
   if (decades==0 || decades>7) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: unsupported decades:%u\n", __FUNCTION__, decades);
      return (ERROR);
   }
   if (listNumber==0 || listNumber>1344) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: unsupported listNumber:%u\n", __FUNCTION__, listNumber);
      return (ERROR);
   }
   if (maxRp==0 || maxRp>2) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: unsupported maxRp:%u\n", __FUNCTION__, maxRp);
      return (ERROR);
   }
   if (maxRc<2 || maxRc>2) { // as now only two variables are supported
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: unsupported maxRc:%u\n", __FUNCTION__, maxRc);
      return (ERROR);
   }
   if (baseRdesc[0]=='\0') {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: empty string 'baseRdesc'\n", __FUNCTION__);
      return (ERROR);
   }
   for (u16 p=0; p<size; p++) {
      //if (dbgLev>=PRINTF) printf ("%g, ", baseR[p]);
      if (baseR[0]==0) {
         if (dbgLev>=PRINTERROR) printf ("ERROR %s: value 0 in 'baseR' not allowed\n", __FUNCTION__);
         return (ERROR);
      }
   }
   return  (OK);
} // int fillConfigVars ()

int main (int numPar, char* param[]) {
   u32  first;
   u08  len, pos;
   u08  p;
   char c;
   u16  numBestRes = NumberResDefault;
   u16  valTy, resTy;
   float allocatedMB;
   int  ret;

   // Use of dynamic memory allocation
   // 0 - load configuration file and params
   ret = fillConfigVars (); // load and check users config file
   if (ret!=OK) {
      if (dbgLev>=PRINTERROR) printf ("ERROR %s: fillConfigVars returned not OK\n", __FUNCTION__);
      return (ERROR);
   }

   // 1 - read and set user request. As now are still partially hard coded
   if (Eseries>0) {
      numR=(u32)Eseries*decades; /* number of existant values of resistance */
      listNumber=0; // not used with Eseries
   } else { /* custom list */
      // listNumber=57; /* insert here custom list quantity */
      // listNumber=473; /* insert here custom list quantity */
      // listNumber=33; /* insert here custom list quantity */
      // listNumber=14; /* insert here custom list quantity */
      // listNumber=20; /* insert here custom list quantity */
      // listNumber=47; /* insert here custom list quantity */
      // listNumber=14; /* insert here custom list quantity */
      // listNumber=120; /* insert here custom list quantity */
      numR=(u32)listNumber; /* number of existant values of resistances */
   }
   //maxRp=1; /* max number of resistances supported per position: as now 1 or 2 */

   // 2 - calculate the needed memory
   if (maxRp==1) {
      numV=numR; /* possible values x each position */
   }
   if (maxRp==2) {
      numV=numR+numR*numR+numR; /* number of possible values x each position */
   }
   //maxRc=2; /* number of resistances (variables) in the circuit: 2 */
   if (maxRc==2) {   /* as now 2 is the only supported value */
      totV=((u32)numV)*((u32)numV); /* number of values to try */
      // compile&doNotStart with 3'882'988'894 max. Start with 917'797'375 max.
      // compile&doNotStart with   268'402'689 max. Start with 59'954'049 max.
      // RunWell                                          with 59'954'049 max.
   }
   valTy = sizeof(struct rValuesTy);
   u32 rValueSize=numV*valTy; // expected no more than '1344x48=64512'
   if (dbgLev>=PRINTDEBUG) printf ("numV:%u X valTy:%u = rValueSize:%u\n", numV, valTy, rValueSize);

   resTy = sizeof(struct resultsTy);
   u64 resultSize=(u64)totV*resTy;
   if (dbgLev>=PRINTDEBUG) printf ("totV:%u X resTy:%u = resultSize:%llu\n", totV, resTy, resultSize);
   //printf ("Will allocate about %f MB of RAM for val\n", rValueSize/1048576.0);
   //printf ("size of struct resultsTy:%u\n", resTy);
   //printf ("Will allocate about %f MB of RAM for res\n", resultSize/1048576.0);
   allocatedMB = (rValueSize/1048576.0+resultSize/1048576.0);

   // 2b - checking arguments syntax
   /*printf ("numPar:%u\n", numPar);*/
   /*printf ("float:%u, double:%u\n", sizeof(float), sizeof (double));*/
   numPar--;
   /*printf ("numPar:%u\n", numPar);*/
   if (numPar==0) { /* no parameters */
      showHelp (allocatedMB);
      return (ERROR);
   }
   if (param[1] != NULL) {
      /*printf ("param1Str:%s\n", param[1]);*/
      desired = atof (param[1]);
      /*printf ("param1Num:%f\n", desired);*/
   }
   if (desired==0.0) desired = DesiredDefault; /* letters case */
   if (numPar>1) { /* at least searched value and formula are present */
      if (param[2] != NULL) {
         /*printf ("param2Str:%s\n", param[2]);*/
         len = strlen (param[2]);
         if (len < 60) {
            pos = strcspn (param[2], "!\"#$&',:;<=>?@[\\]_`{|}~簧");
            if (pos!=len) { // filter out: !"#$&',:;<=>?@[\]_`{|}~簧
               showHelp (allocatedMB);
               printf ("Unsupported char: '%c'\n", *(param[2]+pos));
               return (ERROR);
            }
            for (p=0; p<len; p++) {
               c = *(param[2]+p);
               if ((u08)c>127) {
                  showHelp (allocatedMB);
                  printf ("Unsupported char: '%c'\n", *(param[2]+pos));
                  return (ERROR);
               }
               if (isalpha ((u08)c)) { /* with gcc2.95 is required an integer */
                  if ( (toupper ((u08)c)-'A') >= MaxRc ) {
                              /* ^^^^ old gcc2.95 need int ^^^^ */
                     showHelp (allocatedMB);
                     printf ("ERROR: Unsupported Variable:'%c'\n\n", c);
                     return (ERROR);
                  }
               }
            }
            expr = param[2];
         } else {
            showHelp (allocatedMB);
            printf ("Formula too long: %u chars\n", len);
            return (ERROR);
         }
      }
      if (numBestRes==0) numBestRes = NumberResDefault; /* letters case */
   }
   if (numPar>2) { /* searched value, formula and number of results are present */
      if (param[3] != NULL) {
         /*printf ("param3Str:%s\n", param[3]);*/
         numBestRes = atoi (param[3]);
         /*printf ("param3Num:%u\n", numBestRes);*/
      }
      if (numBestRes==0) numBestRes = NumberResDefault; /* letters case */
      if (numPar>3) printf ("WARNING: Ignoring numPar[%u++]='%s'\n", numPar, param[4]);
   }
   showHead ();

   // 3 - allocate the memory asking to the OS a malloc()
   /* allocated memory for all R: [12*7]+([12*7]*[12*7])+[12*7] : */
   //struct rValuesTy rValues[numV]; /* single, series & parallel */
   //struct rValuesTy* rValues; /* pointer to memory for single, series & parallel */
   printf ("Allocating about %f MB of RAM for inputs ...\n", rValueSize/1048576.0);
   rValues=malloc(rValueSize); // numV*sizeof(struct rValuesTy)
   // allocation fail with 2'930'000'000 on a 4 GB RAM, 8 GB swap, 32 bit OS
   if (!rValues) {
      printf ("Dynamic allocation of:%u Bytes failed:0x%8p\n", rValueSize, rValues);
      return (ERROR);
   }
   // 4 - create the structure's vector inside the allocated memory
   for (u32 e=0; e<numV; e++) {
      rValues[e].rp = (double*) malloc (maxRp*sizeof (double));
      if (!rValues[e].rp) {
         printf ("Dynamic allocation of:%zu Bytes failed:0x%8p\n", maxRp*sizeof (double), rValues[e].rp);
         return (ERROR);
      }
   }
   // now can use global rValues[].rp[], rValues[].r for input values
   printf ("Allocating about %f MB of RAM for results ...\n", resultSize/1048576.0);
   //printf ("size_t size:%u Bytes ptr:%lu\n", sizeof (size_t), (u32) pow(2, 8*sizeof (size_t)));
   if (resultSize > pow(2, 8*sizeof (size_t))) {
      printf ("ERROR: This machine cannot handle allocation of:%llu Bytes\n", resultSize);
      return (ERROR);
   }
   results=malloc(resultSize); // (u64)totV*sizeof(struct resultsTy)
   if (dbgLev>=PRINTDEBUG) printf ("allocated %llu Bytes at:0x%8p\n", resultSize, results);
   if (!results) {
      printf ("Dynamic allocation of:%llu Bytes failed:0x%8p\n", resultSize, results);
      return (ERROR);
   }
   // now can use global results[].pos[], results[].delta for results
   if (dbgLev>=PRINTDEBUG) printf ("Done. Dynamically allocated 'results' at:%p\n", results);
   printf ("Desired Value                  :%11.5f\n", desired);
   printf ("Applied Formula: %s\n", expr);
   exprVarsParser[0]=10.0; exprVarsParser[1]=1.0;
   evalExprParser (expr); /* formula syntax test */
   exprVarsParser[0]=0.0; exprVarsParser[1]=0.0;
   //printf ("sizeof (rValuesTy):%u\n", valTy);
   //printf ("sizeof (rValues[%lu]):%lu\n", numV, val);
   //printf ("sizeof (resultsTy):%u\n", resTy);
   //printf ("sizeof (results[%lu]):%lu\n", totV, res);
   if (Eseries>0) {
      printf ("Resistors Series:E%u, Decades:%u\n", Eseries, decades);
      //printf ("Allocated about %lu MB of RAM\n", 220*powlmy(2, decades-5));
   } else {
      printf ("Resistors custom list of %u values\n", numR);
      /*maxRc = 2;
      maxRp = 1; // as now with custom list of values, we support only 1 resistor
      numR = listNumber;
      numV = listNumber;
      totV = numV * numV;*/
   }
   printf ("Allocated about %f MB of total RAM\n", allocatedMB);

   // 5 - fill the input vectors with needed data
   if (maxRp>1) {
      printf ("Calculating all:%u values with max:%u resistors per position ...\n", numV, maxRp);
   } else {
      printf ("Populating all:%u values with max:%u resistors per position ...\n", numV, maxRp);
   }
   ret = calcRvalues (); /* populate with all possible resistance values */
   if (ret==ERROR) {
      printf ("calcRvalues returned ERROR\n");
      free (results);
      return (ERROR);
   }

   // 6 - calculus of solutions
   printf ("Calculating all:%u solutions with a formula of:%u resistors ...\n", totV, maxRc);
   ret = calcFvalues (); /* calculate all results and delta */
   if (ret==ERROR) {
      printf ("calcFvalues returned ERROR\n");
      free (results);
      return (ERROR);
   }
   if (numBestRes>totV) numBestRes=totV;
   first = totV-numBestRes; /* 0 worse, 'first' 1st printed, 'totV' best, numBestRes # of best print */
//printf ("best:%u results are [%lu-%lu=best]\n", numBestRes, first, totV-1);
//printf ("Print last:%u founded results ...\n", numBestRes);
//ret = showVal (first);

   // 7 - sorting of solutions
   printf ("Sorting all:%u founded solutions ...\n", totV);
   structQuickSort (results, totV); /* QuickSort on all results */

   // 8 - print of results
   printf ("Printing best:%u solutions (top worst, botton best) in all configurations ...\n\n", numBestRes);
   printf ("Show %u solutions with up to 4 resistors:\n", numBestRes);
   ret = showVal (first);
   printf ("\n");
   if (maxRp==1) { // no need to showVal4,3,2 ...
      for (u32 e=0; e<numV; e++) {
         if (rValues[e].rp) free (rValues[e].rp);
      }
      if (results) free (results);
      return (OK);
   }
   printf ("Show %u solutions with 4 resistors:\n", numBestRes);
   ret = showVal4 (numBestRes);
   printf ("\n");
   printf ("Show %u solutions with 3 resistors:\n", numBestRes);
   ret = showVal3 (numBestRes);
   printf ("\n");
   printf ("Show %u solutions with 2 resistors:\n", numBestRes);
   ret = showVal2 (numBestRes);
   printf ("\n");

   // freeing dynamic allocated memory ...
   if (results) free (results);
   for (u32 e=0; e<numV; e++) {
      if (rValues[e].rp) free (rValues[e].rp);
   }
   if (rValues) free (rValues);
   if (baseR) free (baseR);
   if (expr) free (expr);
   return (OK);
}
